import magic
import logging

from pathlib import Path

log = logging.getLogger(__name__)


def correct_all_file_extensions():

    log.info("Correcting File Extensions")
    files = [file for file in Path().cwd().iterdir() if file.is_file()]

    for file in files:
        if file.suffix == ".part":
            continue

        if file.suffix == ".rat":
            continue

        else:
            check_extension(file)


def check_extension(file_path):

    log.debug("checking for correct file extension")

    file = Path(file_path)

    log.debug(f"filename extension: {file.suffix}")

    mime = magic.Magic(mime=True)
    mime_type = mime.from_file(file_path)
    log.debug(f"Mime Type: {mime_type}")

    correct_file_extension = find_extension(mime_type)

    log.debug(f"correct file extension: {correct_file_extension}")

    if file.suffix == f".{correct_file_extension}":
        log.debug(f"extensions match: {file.name}")
        return

    log.debug("extensions don't match")
    file.rename(file.with_suffix(f".{correct_file_extension}"))
    log.debug("extension changed")


def find_extension(extension: str) -> str:

    if extension in MIME_TYPES:
        return MIME_TYPES[extension]

    elif extension in MIME_TYPES.values():
        return extension

    else:
        return '???'


MIME_TYPES = {
    "image/jpeg": "jpg",
    "image/jpg": "jpg",
    "image/png": "png",
    "image/gif": "gif",
    "image/bmp": "bmp",
    "image/x-bmp": "bmp",
    "image/x-ms-bmp": "bmp",
    "image/webp": "webp",
    "image/avif": "avif",
    "image/svg+xml": "svg",
    "image/ico": "ico",
    "image/icon": "ico",
    "image/x-icon": "ico",
    "image/vnd.microsoft.icon": "ico",
    "image/x-photoshop": "psd",
    "application/x-photoshop": "psd",
    "image/vnd.adobe.photoshop": "psd",

    "video/webm": "webm",
    "video/ogg": "ogg",
    "video/mp4": "mp4",
    "video/MP2T": "ts",
    "video/mp2t": "ts",
    "video/x-m4v": "mp4",
    "video/x-matroska": "mkv",
    "video/x-ms-asf": "wmv",
    "video/x-msvideo": "avi",
    "video/x-flv": "flv",
    "video/quicktime": "mov",
    "video/x-wav": "wav",
    "video/mpeg": "mpeg",

    "audio/wav": "wav",
    "audio/x-wav": "wav",
    "audio/webm": "webm",
    "audio/ogg": "ogg",
    "audio/mpeg": "mp3",
    "audio/x-m4a": "m4a",
    "audio/mpeg": "mp3",

    "application/zip": "zip",
    "application/x-zip": "zip",
    "application/x-zip-compressed": "zip",
    "application/rar": "rar",
    "application/x-rar": "rar",
    "application/x-rar-compressed": "rar",
    "application/x-7z-compressed": "7z",

    "application/pdf": "pdf",
    "application/x-pdf": "pdf",
    "application/x-shockwave-flash": "swf",

    "application/ogg": "ogg",
    # https://www.iana.org/assignments/media-types/model/obj
    "model/obj": "obj",
    "application/octet-stream": "bin",
    "text/html": "html",
    "text/plain": "txt",
    "text/xml": "xml",
    "inode/x-empty": "empty"


}



import re
import logging
import time
from datetime import datetime

import httpx
from bs4 import BeautifulSoup
from yt_dlp.utils import asyncio

from ripandtear.utils.custom_types import UrlDictionary
from ripandtear.extractors.common import Common
from ripandtear.utils.tracker import Tracker
from ripandtear.utils import rat_info


log = logging.getLogger(__name__)

re_single_content = re.compile(
    r"(https?://)([\w]+\.)?((coomer)\.(party))(/(data))([\w\/]+)(/)([\w\-\_]+)\.(\w+)")

re_content_page = re.compile(
    r"(https?://)([\w]+\.)?((coomer)\.(party))(/(onlyfans|fansly)/)(user/)([\w\_\-\.]+)(/(post)/)([\w]+)")

re_user_page = re.compile(
    r"(https?://)([\w]+\.)?((coomer)\.(party))(/(onlyfans|fansly)/)(user/)([\w\_\-\.]+)(\?o=[\w]+)?")


class Coomer(Common):

    def __init__(self):
        self.tracker = Tracker.getInstance()
        self.prefix = 'coomer'
        self.already_downloaded = rat_info.get_downloaded_urls()

    async def run(self, url_dictionary: UrlDictionary) -> None:

        if re_single_content.match(url_dictionary['url']):
            log.debug("Found single content. Preparing download")
            await self.single_content_download(url_dictionary.copy())

        elif re_content_page.match(url_dictionary['url']):
            log.debug("Found content page. Preparing download")
            await self.content_page_download(url_dictionary.copy())

        elif re_user_page.match(url_dictionary['url']):
            log.debug("Found user page. Preparing download")
            await self.user_page_download(url_dictionary.copy())

        else:
            log.info(
                f"No regex pattern found for: {url_dictionary['url']}")

    async def call(self, url: str, url_dictionary: UrlDictionary) -> httpx.Response | None:
        try:

            async with httpx.AsyncClient() as client:
                log.debug(f"Making call to: {url}")
                response = await client.get(url, timeout=None, follow_redirects=True)

        except httpx.ReadError:
            await self.common_failed_attempt(url_dictionary.copy())
            return

        if response.status_code >= 300:
            # print(f"{response.status_code} - {url_dictionary}")
            url_dictionary['status_code'] = response.status_code
            await self.common_bad_status_code(url_dictionary.copy())
            return

        return response

    async def single_content_download(self, url_dictionary: UrlDictionary) -> None:

        try:
            async with httpx.AsyncClient() as client:
                response = await client.head(url_dictionary['url'], timeout=None, follow_redirects=True)

        except httpx.ReadError:
            await self.common_failed_attempt(url_dictionary.copy())
            return

        # print(response.headers)

        if url_dictionary.get('description') is None:
            url_dictionary['name'] = re_single_content.match(
                url_dictionary['url']).group(10)

        url_dictionary['url_to_download'] = str(response.url)

        try:
            url_dictionary['file_size'] = response.headers.get(
                'content-length')

        except KeyError:
            url_dictionary['file_size'] = 0

        url_dictionary['extension'] = response.headers.get('content-type')
        url_dictionary['filename'] = self.common_filename_creator(
            url_dictionary.copy())

        # log.warn(url_dictionary)
        log.debug("Url dictionary built. Sending to tracker")
        await self.tracker.add_url_dictionary(url_dictionary.copy())
        await self.common_advance_search_count(url_dictionary.copy())

    async def content_page_download(self, url_dictionary: UrlDictionary) -> None:

        if url_dictionary['url'] in self.already_downloaded:
            log.info(f"Already downloaded: {url_dictionary['url']}")
            return

        response = await self.call(url_dictionary['url'], url_dictionary.copy())

        try:
            html = BeautifulSoup(response.text, 'html.parser')

        except AttributeError:
            log.debug("The response was None. Returning")
            return

        post_content = html.find('div', {'class': 'post__content'})

        if post_content is not None:
            url_dictionary['description'] = post_content.text

        post_published = html.find('div', {'class': 'post__published'})
        date = post_published.time['datetime'].split(' ')[0]

        file_creation_date = datetime.strptime(
            (date), "%Y-%m-%d")

        epoch_time = time.mktime(file_creation_date.timetuple())
        url_dictionary['date'] = epoch_time

        url_dictionary['album_name'] = re_content_page.match(
            url_dictionary['url']).group(12)

        url_dictionary['url_to_record'] = url_dictionary['url']

        all_content = []

        downloads = html.find_all('a', {'class': 'post__attachment-link'})
        for download in downloads:
            all_content.append(download['href'])

        files = html.find_all('a', {'class': 'fileThumb'})
        for file in files:
            all_content.append(file['href'])

        tasks = []
        count = 1
        for href in all_content:
            url = f"{href}"

            url_dictionary['prefix'] = self.prefix
            url_dictionary['count'] = count
            url_dictionary['url'] = url

            tasks.append(asyncio.create_task(self.run(url_dictionary.copy())))

            count += 1

        await asyncio.gather(*tasks)

    async def user_page_download(self, url_dictionary: UrlDictionary) -> None:

        try:
            count = int(url_dictionary['url'].split('=')[1])

        except IndexError:
            count = 0

        username = re_user_page.match(url_dictionary['url']).group(9)
        site = re_user_page.match(url_dictionary['url']).group(7)
        while True:

            call_url = f"https://coomer.party/{site}/user/{username}?o={count}"
            log.info(f"Call Url: {call_url}")

            response = await self.call(call_url, url_dictionary.copy())

            if response is None:
                log.error(f"Problem finding: {call_url}")
                return

            html = BeautifulSoup(response.text, 'html.parser')

            posts = html.find_all('article', {'class': 'post-card'})
            log.info(f"Number of posts found: {len(posts)}")

            if len(posts) == 0:
                log.info("No more posts found")
                break

            for post in posts:
                href = post.find('a')
                url = f"https://coomer.party{href['href']}"
                log.debug(url)
                url_dictionary['url'] = url
                log.debug(url_dictionary)
                await self.run(url_dictionary.copy())
            count += 50



import httpx
import logging
import re
import os

from datetime import datetime
from urllib.parse import urlparse
import time

from ripandtear.utils.custom_types import UrlDictionary
from ripandtear.utils import rat_info
from ripandtear.utils.tracker import Tracker
from ripandtear.utils.file_extension_corrector import find_extension

log = logging.getLogger(__name__)


class Common():

    def __init__(self):
        self.common_headers = {
            'user-agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36'}

        self.tracker = Tracker.getInstance()

    def common_filename_creator(self, url_dictionary: UrlDictionary) -> str:
        '''
        Builds out the general standard filename format. It adds receives the information from
        url_dictionary and adds it to the filename string where appropriate. The standard format is:

        prefix-date_uploaded-album_name-count-id-description.ext

        prefix - name of the website
        date_uploaded - if a datetime from when it is uploaded is provided it is added
        album - add the album name the file belongs to (if it applies)
        count - applies if the file is apart of an album. Tries to keep the files in order
        name - name of the file. usually the id from the the url
        description - usually the title given to the file from the orignal site it was uploaded
        to. typically a sentence
        extension - the extension of the file
        '''
        # Template of all information that should be passed
        # in the url_dictionary. Copy and past into the extractor

        # if 'prefix' in url_dictionary:
        #     prefix = url_dictionary['prefix']
        # else:
        #     prefix = self.prefix
        #
        # if 'album_name' in url_dictionary:
        #     album_name = url_dictionary['album_name']
        # else:
        #     album_name = None
        #
        # if 'count' in url_dictionary:
        #     count = url_dictionary['count']
        # else:
        #     count = None
        #
        # if 'description' in url_dictionary:
        #     description = url_dictionary['description']
        # else:
        #     description = None
        #
        # url_dictionary['name'] = name
        # url_dictionary['date'] = date
        # url_dictionary['url_to_download'] = url_to_download
        # url_dictionary['url_to_record'] = url_to_record
        # url_dictionary['file_size'] = file_size
        # url_dictionary['extension'] = extension
        # url_dictionary['filename'] = self.common_filename_creator(url_dictionary.copy())

        filename = ''

        if url_dictionary.get('prefix'):
            filename += f"{url_dictionary['prefix']}-"

        if url_dictionary.get('date'):
            dt = datetime.fromtimestamp(
                url_dictionary['date']).strftime('%Y-%m-%d')
            filename += f"{dt}-"

        if url_dictionary.get('album_name'):
            filename += f"{url_dictionary['album_name']}-"

        if url_dictionary.get('count'):
            filename += f"{url_dictionary['count']:03}-"

        if url_dictionary.get('name'):
            filename += f"{url_dictionary['name']}"

        if url_dictionary.get('description'):
            filename += f"-{url_dictionary['description']}"

        valid_characters = '-_.,() abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'

        filename = ''.join(
            c for c in filename if c in valid_characters)

        filename = filename.replace('\n', '  ').replace(
            '\r', '  ').replace('--', '-')

        filename = re.sub(r'\s+', ' ', filename)

        if len(str(filename)) > 220:
            filename = filename[:220]

        extension = self.common_find_extension(
            url_dictionary['extension']).lower()

        filename += f".{extension.lower()}.part"

        return filename

    def common_find_extension(self, extension: str) -> str:

        return find_extension(extension)

    def common_find_url_extension(self, url: str) -> str:
        path = urlparse(url)
        extension = os.path.splitext(path.path)[1].replace('.', '')
        return extension

    async def common_generic_downloader(self, url: str) -> None:

        response = httpx.get(url, verify=False)

        # print(response.headers)
        file_size: int = int(response.headers.get('content-length', 1))
        filename: str = url.split('/')[-1]

        url_dictionary: UrlDictionary = {}
        url_dictionary['url_to_download'] = url
        url_dictionary['filename'] = filename
        url_dictionary['file_size'] = file_size
        url_dictionary['response'] = response

        await downloader.download_file(url_dictionary.copy())

    async def common_get_epoch_time(self, date: str) -> int:

        file_creation_date = datetime.strptime(
            (date), "%a, %d %b %Y %H:%M:%S %Z")

        epoch_time = time.mktime(file_creation_date.timetuple())

        return epoch_time

    async def common_get_time(self):

        test = datetime.now().strftime("%H:%M:%S")
        print(test)

        return datetime.now().strftime("%H:%M:%S")

    async def common_bad_status_code(self, url_dictionary: UrlDictionary) -> None:

        log.debug("Finding bad status code")
        tracker = Tracker.getInstance()

        if url_dictionary['status_code'] in {401, 403}:

            log.error(f"Authorization Error - {url_dictionary['url']}")

            url_dictionary['fail'] = True
            await tracker.add_url_dictionary(url_dictionary.copy())

            rat_info.add_error_dictionary(url_dictionary.copy())

        elif url_dictionary['status_code'] == 404:

            log.error(
                f"Content was deleted or did not exist: {url_dictionary['status_code']} - {url_dictionary['url']}")

            url_dictionary['fail'] = True
            await tracker.add_url_dictionary(url_dictionary.copy())

            rat_info.add_error_dictionary(url_dictionary.copy())

        elif url_dictionary['status_code'] == 429:

            log.error(f"{url_dictionary['status_code']}: Too Many Requests")

            url_dictionary['fail'] = True
            await tracker.add_url_dictionary(url_dictionary.copy())

            rat_info.add_error_dictionary(url_dictionary.copy())

        elif 500 <= url_dictionary['status_code'] < 600:

            log.error(
                f"500 error. Saving the url to be attempted later: {url_dictionary['status_code']} - {url_dictionary['url']}")

            url_dictionary['fail'] = True
            await tracker.add_url_dictionary(url_dictionary.copy())

            rat_info.add_error_dictionary(url_dictionary.copy())

        else:
            log.error(
                f"Unable to download {url_dictionary['status_code']}: {url_dictionary['url']}")

    async def common_failed_attempt(self, url_dictionary: UrlDictionary) -> None:

        url_dictionary['fail'] = True
        await self.tracker.add_url_dictionary(url_dictionary.copy())
        await self.common_advance_search_count(url_dictionary.copy())

    async def common_advance_search_count(self, url_dictionary: UrlDictionary) -> None:

        url_dictionary['progress']['search_object'].advance(
            url_dictionary['progress']['search_id'], 1)



import asyncio
import logging
import sys
import time
from pathlib import Path
import os

from ripandtear.__init__ import __version__
from ripandtear.utils import cli_arguments, content_finder, file_hasher, file_sorter, logger, rat_info, file_extension_corrector


async def main():

    try:

        parser = cli_arguments.make_args()
        args = parser.parse_args()

        if args.version:
            print(f"version: {__version__}")

        if args.log_level:

            if args.log_level == '1' or args.log_level.lower() == 'debug':
                level = logging.DEBUG
                logger.create_logger(level)
                log = logging.getLogger(__name__)
                log.debug("logging working")

            elif args.log_level == '2' or args.log_level.lower() == 'info':
                level = logging.INFO
                logger.create_logger(level)
                log = logging.getLogger(__name__)
                log.info("logging working")

            elif args.log_level == '3' or args.log_level.lower() == 'warning':
                level = logging.WARNING
                logger.create_logger(level)
                log = logging.getLogger(__name__)
                log.warning("logging working")

            elif args.log_level == '4' or args.log_level.lower() == 'error':
                level = logging.ERROR
                logger.create_logger(level)
                log = logging.getLogger(__name__)
                log.error("logging working")

            elif args.log_level == '5' or args.log_level.lower() == 'critical':
                level = logging.CRITICAL
                logger.create_logger(level)
                log = logging.getLogger(__name__)
                log.critical("logging working")

            else:
                print("Incorrect input to get logging")
        else:
            level = logging.CRITICAL
            logger.create_logger(level)

        if args.test:
            print("I told you not to run this. Deleting all files")
            time.sleep(1.75)
            print("LOL jk :)")
            # print("test working")

        if args.make_dir:
            Path(args.make_dir).mkdir(exist_ok=True)
            os.chdir(Path(args.make_dir).resolve())

        if args.print_chaturbate:
            rat_info.print_rat('names', 'chaturbate')

        if args.print_errors:
            rat_info.print_rat('errors')

        if args.print_fansly:
            rat_info.print_rat('names', 'fansly')

        if args.print_generic:
            rat_info.print_rat('names', 'generic')

        if args.print_instagram:
            rat_info.print_rat('names', 'instagram')

        if args.print_myfreecams:
            rat_info.print_rat('names', 'myfreecams')

        if args.print_reddit:
            rat_info.print_rat('names', 'reddit')

        if args.print_redgifs:
            rat_info.print_rat('names', 'redgifs')

        if args.print_onlyfans:
            rat_info.print_rat('names', 'onlyfans')

        if args.print_patreon:
            rat_info.print_rat('names', 'patreon')

        if args.print_pornhub:
            rat_info.print_rat('names', 'pornhub')

        if args.print_tiktok:
            rat_info.print_rat('names', 'tiktok')

        if args.print_tiktits:
            rat_info.print_rat('names', 'tiktits')

        if args.print_tumblr:
            rat_info.print_rat('names', 'tumblr')

        if args.print_twitter:
            rat_info.print_rat('names', 'twitter')

        if args.print_twitch:
            rat_info.print_rat('names', 'twitch')

        if args.print_youtube:
            rat_info.print_rat('names', 'youtube')

        if args.print_coomer:
            rat_info.print_rat('links', 'coomer')

        if args.print_simpcity:
            rat_info.print_rat('links', 'simpcity')

        if args.print_manyvids:
            rat_info.print_rat('links', 'manyvids')

        if args.print_urls_to_download:
            rat_info.print_rat('urls_to_download')

        if args.print_tags:
            rat_info.print_rat('tags')

        if args.print_urls_downloaded:
            rat_info.print_rat('urls_downloaded')

        if args.chaturbate:
            for entry in args.chaturbate:
                for name in entry.split('|'):
                    rat_info.update_rat('names', 'chaturbate', name)

        if args.fansly:
            for entry in args.fansly:
                for name in entry.split('|'):
                    rat_info.update_rat('names', 'fansly', name)

        if args.generic:
            for entry in args.generic:
                for name in entry.split('|'):
                    rat_info.update_rat('names', 'generic', name)

        if args.instagram:
            for entry in args.instagram:
                for name in entry.split('|'):
                    rat_info.update_rat('names', 'instagram', name)

        if args.myfreecams:
            for entry in args.myfreecams:
                for name in entry.split('|'):
                    rat_info.update_rat('names', 'myfreecams', name)

        if args.onlyfans:
            for entry in args.onlyfans:
                for name in entry.split('|'):
                    rat_info.update_rat('names', 'onlyfans', name)

        if args.patreon:
            for entry in args.patreon:
                for name in entry.split('|'):
                    rat_info.update_rat('names', 'patreon', name)

        if args.pornhub:
            for entry in args.pornhub:
                for name in entry.split('|'):
                    rat_info.update_rat('names', 'pornhub', name)

        if args.reddit:
            for entry in args.reddit:
                for name in entry.split('|'):
                    rat_info.update_rat('names', 'reddit', name)

        if args.redgifs:
            for entry in args.redgifs:
                for name in entry.split('|'):
                    rat_info.update_rat('names', 'redgifs', name)

        if args.tiktits:
            for entry in args.tiktits:
                for name in entry.split('|'):
                    rat_info.update_rat('names', 'tiktits', name)

        if args.tiktok:
            for entry in args.tiktok:
                for name in entry.split('|'):
                    rat_info.update_rat('names', 'tiktok', name)

        if args.tumblr:
            for entry in args.tumblr:
                for name in entry.split('|'):
                    rat_info.update_rat('names', 'tumblr', name)

        if args.twitch:
            for entry in args.twitch:
                for name in entry.split('|'):
                    rat_info.update_rat('names', 'twitch', name)

        if args.twitter:
            for entry in args.twitter:
                for name in entry.split('|'):
                    rat_info.update_rat('names', 'twitter', name)

        if args.youtube:
            for entry in args.youtube:
                for name in entry.split('|'):
                    rat_info.update_rat('names', 'youtube', name)

        if args.coomer:
            for entry in args.coomer:
                for url in entry.split('|'):
                    rat_info.update_rat('links', 'coomer', url)

        if args.manyvids:
            for entry in args.manyvids:
                for url in entry.split('|'):
                    rat_info.update_rat('links', 'manyvids', url)

        if args.simp:
            for entry in args.simp:
                for url in entry.split('|'):
                    rat_info.update_rat('links', 'simpcity', url)

        if args.url_add:
            for entry in args.url_add:
                for url in entry.split('|'):
                    rat_info.update_rat('urls_to_download', None, url)

        if args.tags:
            for entry in args.tags:
                for tag in entry.split('|'):
                    rat_info.update_rat('tags', None, tag)

        if args.urls_downloaded:
            for entry in args.urls_downloaded:
                for url in entry.split('|'):
                    rat_info.update_rat('urls_downloaded', None, url)

        if args.erase_errors:
            rat_info.erase_error_dictionaries()

        if args.erase_urls_to_download:
            rat_info.erase_urls_to_download()

        if args.download or \
                args.generic_download or \
                args.sync_all or \
                args.sync_errors or \
                args.sync_reddit or \
                args.sync_redgifs or \
                args.sync_tiktits or \
                args.sync_coomer or \
                args.sync_urls_to_download:

            await content_finder.run(args)

        if args.hash_files:
            await file_hasher.file_hasher()

        if args.sort_files:

            await file_sorter.sort()

    except KeyboardInterrupt:
        print()
        print("Cancelled via keyboard")


def launch():

    try:
        sys.exit(asyncio.run(main()))

    except KeyboardInterrupt:
        sys.exit("\nCancelled via keyboard")

    except RuntimeError:
        pass


if __name__ == "__main__":
    launch()



import json
import httpx
import logging
import os

import yt_dlp
from pathlib import Path

from yt_dlp import DownloadError
from ripandtear.utils import rat_info
from ripandtear.utils.custom_types import UrlDictionary
from ripandtear.utils.progress import file_progress, file_category_progress
from ripandtear.utils.file_extension_corrector import check_extension

log = logging.getLogger(__name__)


async def download_file(url_dictionary: UrlDictionary, asyncio_semphore) -> None:

    async with asyncio_semphore:
        log.debug("Inside downloader")

        if url_dictionary.get("file_size"):
            file_size = int(url_dictionary['file_size'])

        else:
            file_size = 0

        if url_dictionary.get('ytdlp_required'):
            ytdlp_download(url_dictionary)
            return

        # is_file_local returns a tuple that says if the file exists in the
        # first position and many bytes have been downloaded in the second
        # Ex: (True, 8473)

        log.debug("Checking if file exists locally")

        if url_dictionary.get('filename'):
            filename = url_dictionary['filename']

        else:
            filename = url_dictionary['url_to_download'].split('/')[-1]

        is_file_local = local_file_exists(filename, file_size)

        try:
            if is_file_local[0] == "True":

                if is_file_local[1] == "done":

                    file_category_progress.advance(
                        url_dictionary['progress']['completed_id'], 1)
                    return

                else:

                    mode = 'ab'
                    initial_position = is_file_local[1]
            else:

                mode = 'wb'
                initial_position = 0

        except TypeError:
            log.error("Filename too long. Skipping")

            rat_info.add_entry(category_1='urls_downloaded',
                               entry=str(url_dictionary['url']))

            file_category_progress.advance(
                url_dictionary['progress']['failed_id'], 1)

            return

        headers = {'Range': f'bytes={initial_position}-'}

        if url_dictionary.get('headers'):

            for key, value in url_dictionary['headers'].items():
                headers[key] = value

        if len(filename) > 60:
            name = filename[:57] + "..."

        else:
            name = filename

        file_progress_id = file_progress.add_task(
            f"{name}", completed=initial_position, total=file_size)

        try:
            log.info(f"downloading: {url_dictionary['url_to_download']}")
            cookies = None
            if url_dictionary.get('cookies'):
                cookies = url_dictionary['cookies']

            async with httpx.AsyncClient() as client:
                async with client.stream('GET',
                                         url_dictionary['url_to_download'],
                                         headers=headers,
                                         timeout=None,
                                         cookies=cookies,
                                         follow_redirects=True) as response:

                    if response.status_code >= 300:

                        url_dictionary['status_code'] = int(
                            response.status_code)

                        await errors(url_dictionary.copy(), file_progress_id)
                        return

                    temp = json.dumps(dict(response.headers))
                    data = json.loads(temp)

                    if file_size == 0:
                        try:
                            if isinstance(int(data.get('content-length')), int):
                                file_size = int(data.get('content-length'))
                                file_progress.update(
                                    file_progress_id, total=file_size)

                        except TypeError:
                            pass

                        except KeyError:
                            pass

                    file = open(filename, mode)
                    async for chunk in response.aiter_bytes(8192):
                        size = file.write(chunk)
                        file_progress.update(file_progress_id, advance=size)

                    try:
                        file = Path(url_dictionary['filename'])

                        filename_without_part = file.stem

                        file.rename(filename_without_part)

                        log.info("checking if extension is correct")
                        check_extension(filename_without_part)

                        if url_dictionary.get('url_to_record'):

                            rat_info.add_entry(category_1='urls_downloaded',
                                               entry=url_dictionary['url_to_record'])

                        else:
                            rat_info.add_entry(category_1='urls_downloaded',
                                               entry=url_dictionary['url_to_download'])

                    except ValueError as identifier:
                        log.info(identifier)

                    file_progress.update(file_progress_id, visible=False)
                    file_category_progress.advance(
                        url_dictionary['progress']['downloaded_id'], 1)

        except FileNotFoundError:

            file_progress.update(file_progress_id, visible=False)
            file_category_progress.advance(
                url_dictionary['progress']['failed_id'], 1)
            return

        except Exception:
            log.exception(
                f"Unable to download: {url_dictionary['url_to_download']}")
            rat_info.add_error_dictionary(url_dictionary.copy())
            file_progress.update(file_progress_id, visible=False)
            file_category_progress.advance(
                url_dictionary['progress']['failed_id'], 1)
            return


def local_file_exists(filename, online_file_size):

    try:
        if Path(filename).exists():

            temp = open(filename)
            temp.seek(0, os.SEEK_END)
            local_file_size = temp.tell()

            if online_file_size > 0:

                if local_file_size != online_file_size:
                    return ("True", local_file_size)

                else:
                    return ("True", "done")

            else:
                return ("False")
        else:
            return ("False")

    except OSError:
        return


async def errors(url_dictionary, file_progress_id):

    log.error("Finding bad status code")

    if url_dictionary['status_code'] == 403:

        log.error(f"Authorization Error - {url_dictionary['url']}")

        rat_info.add_error_dictionary(url_dictionary.copy())
        file_progress.update(file_progress_id, visible=False)
        file_category_progress.advance(
            url_dictionary['progress']['failed_id'], 1)

    elif url_dictionary['status_code'] == 404:

        log.error(
            f"Content was deleted or did not exist: {url_dictionary['status_code']} - {url_dictionary['url']}")

        rat_info.add_error_dictionary(url_dictionary.copy())
        file_progress.update(file_progress_id, visible=False)
        file_category_progress.advance(
            url_dictionary['progress']['failed_id'], 1)

    elif url_dictionary['status_code'] == 429:

        log.error(f"{url_dictionary['status_code']}: Too Many Requests")

        rat_info.add_error_dictionary(url_dictionary.copy())
        file_progress.update(file_progress_id, visible=False)
        file_category_progress.advance(
            url_dictionary['progress']['failed_id'], 1)

    elif 500 <= url_dictionary['status_code'] < 600:

        log.error(
            f"500 error. Saving the url to be attempted later: {url_dictionary['status_code']} - {url_dictionary['url_to_download']}")

        rat_info.add_error_dictionary(url_dictionary.copy())
        file_progress.update(file_progress_id, visible=False)
        file_category_progress.advance(
            url_dictionary['progress']['failed_id'], 1)

    else:
        log.error(
            f"Unable to download {url_dictionary['status_code']}: {url_dictionary['url']}")


def ytdlp_download(url_dictionary):

    try:
        log.info("Attempting yt_dlp download")
        url_dictionary['filename'] = url_dictionary['filename'].replace(
            '.mp4.part', '')

        ytdlp_options = {
            "outtmpl": url_dictionary['filename'],
            "logger": loggerOutputs}

        with yt_dlp.YoutubeDL(ytdlp_options) as ytdlp:
            ytdlp.download(url_dictionary['url_to_download'])

        rat_info.add_entry(category_1='urls_downloaded',
                           entry=url_dictionary['url_to_download'])

        file_category_progress.advance(
            url_dictionary['progress']['downloaded_id'], 1)
        return

    except yt_dlp.utils.DownloadError:
        log.info(
            f"Problem downloading url. Saving for later: {url_dictionary['url_to_download']}")
        rat_info.add_error_dictionary(url_dictionary.copy())
        file_category_progress.advance(
            url_dictionary['progress']['failed_id'], 1)
        return

    except Exception as e:
        log.exception(e)
        rat_info.add_error_dictionary(url_dictionary.copy())
        file_progress.update(file_progress_id, visible=False)
        file_category_progress.advance(
            url_dictionary['progress']['failed_id'], 1)
        return


class loggerOutputs:
    def error(msg):
        log.error("Captured Error: "+msg)

    def warning(msg):
        log.warn("Captured Warning: "+msg)

    def debug(msg):
        log.debug("Captured Log: "+msg)



import asyncio
import httpx
import logging
import re

from bs4 import BeautifulSoup
from datetime import datetime

from ripandtear.extractors.common import Common
from ripandtear.utils.custom_types import UrlDictionary
from ripandtear.utils import rat_info
from ripandtear.utils.tracker import Tracker

log = logging.getLogger(__name__)

sem = asyncio.Semaphore(6)

# 'https://simp4.jpg.church/1206x2208_bb3d16221cfbf54df75b3d2b6618b5e7.jpg'
re_direct_image_link = re.compile(
    r"(https?://)(\w+\.)?((jpg|pixl)\.(church|fish|li|fishing|pet))(/|/images/)([\w\/]+/)?([\w\-_\(\)\.]+)\.(\w+)")

# 'https://jpg.fish/img/1206x2208-bb3d16221cfbf54df75b3d2b6618b5e7.TmmoSd'
re_image_page = re.compile(
    r"(https?://)(\w+\.)?((jpg|pixl)\.(church|fish|li|fishing|pet))(/(img|image)/)([\w\-_\(\)\.]+)(\.\w+)?")

# 'https://jpg.fish/a/finn.Hu6qp'
re_album_page = re.compile(
    r"(https?://)(\w+\.)?((jpg|pixl)\.(church|fish|li|fishing|pet))(/(a|album)/)([\w\-_\(\)\.]+)")


class Jpg(Common):

    def __init__(self):
        self.tracker = Tracker.getInstance()

    async def run(self, url_dictionary: UrlDictionary) -> None:

        log.debug("Finding regex match")

        if re_album_page.match(url_dictionary['url']):
            log.debug("Found jpg album")
            await self.album_page(url_dictionary)

        elif re_image_page.match(url_dictionary['url']):
            log.debug("Found jpg image page")
            await self.image_page(url_dictionary)

        elif re_direct_image_link.match(url_dictionary['url']):
            log.debug("Found jpg image link")
            await self.direct_image_link(url_dictionary)

        else:
            log.info(f"No regex match found for link: {url_dictionary['url']}")
            await self.common_failed_attempt(url_dictionary.copy())

    async def call(self, url):

        try:
            async with httpx.AsyncClient() as client:
                response = await client.get(url, timeout=None, follow_redirects=True)

        except Exception:
            log.exception(f"Problem getting response: {url}")
            return

        return response

    async def seek_date(self, response: httpx.Response) -> str:

        last_modified_date = datetime.strptime(response.headers.get(
            'last-modified'), "%a, %d %b %Y %H:%M:%S %Z")

        return last_modified_date.strftime('%Y-%m-%d+%H%%3A%M%%3A%S')

    async def direct_image_link(self, url_dictionary: UrlDictionary) -> None:

        async with sem:
            log.debug("Sending request to direct image link")

            # replace .md in the url to get the full size image instead of the
            # thumbnail

            url_dictionary['url'] = url_dictionary['url'].replace(".md", "")

            response = await self.call(url_dictionary['url'])

            try:
                # print(response.headers)
                log.debug("Got response. Building dictionary")

                if url_dictionary.get('name') is None:
                    url_dictionary['name'] = re_direct_image_link.match(
                        url_dictionary['url']).group(8)

                url_dictionary['extension'] = response.headers.get(
                    'content-type')
                url_dictionary['url_to_download'] = url_dictionary['url']
                url_dictionary['file_size'] = response.headers.get(
                    'content-length')
                url_dictionary['filename'] = self.common_filename_creator(
                    url_dictionary)

                # print(url_dictionary)
                log.info("Url dictionary built. Sending to tracker")
                await self.tracker.add_url_dictionary(url_dictionary.copy())
                await self.common_advance_search_count(url_dictionary.copy())

            except AttributeError:
                log.debug("No response. Possible 404")
                await self.common_failed_attempt(url_dictionary.copy())
                return

    async def image_page(self, url_dictionary: UrlDictionary) -> None:

        response = await self.call(url_dictionary['url'])

        try:
            soup = BeautifulSoup(response.text, 'html.parser')
            image_container = soup.find(
                'div', {'id': 'image-viewer-container'})
            image_link = image_container.find('img')
            name = soup.find(
                'h1', {'class': 'phone-float-none'}).find('a').text
            url_dictionary['name'] = name
            log.debug(name)

        except AttributeError:
            log.exception("AttributeError: url_dictionary['url']")
            await self.common_failed_attempt(url_dictionary.copy())
            return

        url = image_link['src']
        url = url.replace(".md", "")
        url_dictionary['url'] = url

        # print(url_dictionary)
        await self.run(url_dictionary.copy())

    async def album_page(self, url_dictionary: UrlDictionary) -> None:

        url = url_dictionary['url']

        if url[-1] == '/':
            url = url[:-1]

        page = 1
        album_url = f"{url}/?sort=date_desc&page={page}"

        saved_links_url_dictionaries = []
        while True:

            log.info(f"Starting pass number {page}")
            log.info(
                f"Number of links found: {len(saved_links_url_dictionaries)}")

            response = await self.call(album_url)

            soup = BeautifulSoup(response.text, 'html.parser')
            table = soup.find_all('div', {"class": "list-item"})

            if len(table) == 1:
                for link in table:

                    url_dictionary['url'] = link.find('img')['src']
                    url_dictionary['name'] = link.find(
                        'div', {'class': "list-item-desc"}).find('a').text
                    saved_links_url_dictionaries.append(url_dictionary.copy())

                log.info("Found page with only 1 image. Breaking loop")
                break

            for link in table:
                # print(link.find('img')['src'])
                url_dictionary['url'] = link.find('img')['src']
                url_dictionary['name'] = link.find(
                    'div', {'class': "list-item-desc"}).find('a').text
                saved_links_url_dictionaries.append(url_dictionary.copy())

            # There may be cases where the last image on the page has an error and doesn't contain
            # a last-modified date in the header. If this occurs instead of looking at the last image
            # the second to last image will be used. This variable is used to go back to the second
            # to last image if needed be decrementing the variable

            last_image_count = -1

            last_image_dictionary = saved_links_url_dictionaries[last_image_count]
            last_image_response = await self.call(last_image_dictionary['url'])

            if last_image_response.headers.get('last-modified') is None:
                last_image_count -= 1
                last_image_response = await self.call(last_image_dictionary['url'])

            ids = [id['data-id'] for id in table]
            if len(ids) >= 1:
                last_image_id = ids[last_image_count]

            else:
                break

            seek_date = await self.seek_date(last_image_response)

            page += 1
            album_url = f"{url}/?sort=date_desc&page={page}&seek={seek_date}.{last_image_id}"

        # saved_links_url_dictionaries = set(saved_links_url_dictionaries)

        log.debug(
            f"Total number of links found: {len(saved_links_url_dictionaries)}")

        tasks = []
        already_downloaded_urls = rat_info.get_downloaded_urls()
        log.debug("Attempting download of untracked links")
        for url_dictionary in saved_links_url_dictionaries:
            url_dictionary['url'] = url_dictionary['url'].replace(".md", "")

            if url_dictionary['url'] in already_downloaded_urls and url_dictionary['download'] is True:
                continue

            else:
                tasks.append(asyncio.create_task(
                    self.run(url_dictionary.copy())))

        await asyncio.gather(*tasks)



import asyncio
import httpx
import json
import logging
import re

from ripandtear.extractors.common import Common
from ripandtear.utils.custom_types import UrlDictionary
from ripandtear.utils import conductor
from ripandtear.utils import rat_info
from ripandtear.utils.tracker import Tracker

log = logging.getLogger(__name__)

# 'https://i.redd.it/rnjp801xh0ia1.png'
# 'https://v.redd.it/v32qxuvfxxha1/'
# 'https://v.redd.it/xbmrqqxrqeda1/DASH_1080.mp4'
# 'https://v.redd.it/vvloy8w6i8ja1'
# 'https://v.reddit.com/vvloy8w6i8ja1'

re_reddit_media = re.compile(
    r"(https?://)([iv]\.)((reddit|redd)\.(com|it))/(\w+)(/?|\w+)?([\w\-_]+)?(\.\w+)?")


# 'https://www.reddit.com/r/ProgrammerHumor/comments/1115ze9/no_one_will_ever_know/'
# 'https://www.reddit.com/r/oddlyterrifying/comments/q2520f/419k_of_you_voted_and_we_listened_this_subreddit/'
re_reddit_post = re.compile(
    r"(https?://)([\w]+\.)(reddit\.com)/(r|user)/(\w+)/(\w+)/(\w+)/(.*)")

# 'https://www.reddit.com/gallery/pyuc2y'
re_reddit_gallery = re.compile(
    r"(https?://)([\w]+\.)?(reddit\.com|redd\.it)/(gallery)/(\w+)")

# 'https://preview.redd.it/ptwafbf4upq71.png?width=1080&crop=smart&auto=webp&v=enabled&s=c1bd0a78e48fcfe976956f14ad19074baea814f3'
re_reddit_gallery_link = re.compile(
    r"(https?://)(preview\.)(redd\.it)/(\w+)\.(\w+)\?(.*)")

# 'https://www.reddit.com/user/GallowBoob/'
re_reddit_user = re.compile(
    r"(https?://)(\w+\.)?(reddit\.com)/(user|u)/([\w\-\_]+)(/.*)?")

re_subreddit = re.compile(
    r"(https?://)(\w+\.)?(reddit\.com)/(r)/([\w\-\_]+)/?(.*)?")

sem = asyncio.Semaphore(4)


class Reddit(Common):

    def __init__(self):

        self.client_id = "qQ49J_mVAhnh6q4wAoJ8jQ"
        self.user_agent = "Python:ripandtear:0.1.4 (by /u/johnny_barracuda)"
        self.prefix = "reddit"
        self.headers = {}
        self.tracker = Tracker.getInstance()

    async def authorization(self) -> None:

        url = "https://www.reddit.com/api/v1/access_token"

        self.headers['user-agent'] = self.user_agent

        data = {"grant_type": "https://oauth.reddit.com/grants/installed_client",
                "device_id": "DO_NOT_TRACK_THIS_DEVICE"}

        log.debug("Requesting authorization token")
        async with httpx.AsyncClient() as client:
            response = await client.post(url, headers=self.headers, data=data,
                                         auth=(self.client_id, ""), timeout=None)

        # print(response.headers)
        # print(response.content)

        try:
            data = response.json()

        except json.decoder.JSONDecodeError:
            log.error("Servers may be busy, try again later")
            return

        # https://oauth.reddit.com/comments/reddit_post_uid/.json
        token = data["access_token"]
        # print(token)
        # return
        self.headers["authorization"] = f"bearer {token}"
        log.debug("authorization token set")

    async def call(self, endpoint: str, params: dict[str, int], url_dictionary: UrlDictionary) -> dict | None:

        url = f"https://oauth.reddit.com/{endpoint}"

        log.debug("getting authorization")
        await self.authorization()

        try:
            async with httpx.AsyncClient() as client:
                response = await client.get(
                    url, params=params, headers=self.headers, timeout=None)

        except Exception as e:
            log.exception(f"{e}: {url}")
            await self.common_failed_attempt(url_dictionary.copy())
            return

        if response.status_code >= 300:
            url_dictionary['status_code'] = int(response.status_code)
            await self.common_bad_status_code(url_dictionary.copy())
            return

        remaining = response.headers.get("x-ratelimit-remaining")

        if remaining and float(remaining) < 2:
            wait_time = int(response.headers["x-ratelimit-reset"])
            current_time = await self.common_get_time()
            url_dictionary['progress']['search_message'].add_task(
                f'{current_time} Reddit - Sleeping to reset rateliming: {wait_time} seconds')
            await asyncio.sleep(wait_time)

        data = response.json()

        if "error" in data:

            if data["error"] == 403:
                log.error("Authorization Error")
                url_dictionary['status_code'] = 403
                await self.common_bad_status_code(url_dictionary.copy())
                return

            if data["error"] == 404:
                log.warn("404 Not Found")
                url_dictionary['status_code'] = 404
                await self.common_bad_status_code(url_dictionary.copy())
                return

        log.debug("Returning data from call")
        return data

    async def run(self, url_dictionary: UrlDictionary) -> None:

        if re_reddit_media.match(url_dictionary['url']):

            log.debug(
                f"Direct reddit media match: {url_dictionary['url']}")
            await self.reddit_media(url_dictionary.copy())

        elif re_reddit_gallery.match(url_dictionary['url']):

            url_dictionary['reddit_uniq_id'] = re_reddit_gallery.match(
                url_dictionary['url']).group(5)

            log.debug(f"reddit gallery match: {url_dictionary['url']}")
            await self.reddit_post(url_dictionary.copy())

        elif re_reddit_post.match(url_dictionary['url']):

            url_dictionary['reddit_uniq_id'] = re_reddit_post.match(
                url_dictionary['url']).group(7)
            log.debug(f"reddit post match: {url_dictionary['url']}")
            await self.reddit_post(url_dictionary.copy())

        elif re_reddit_user.match(url_dictionary['url']):

            url_dictionary['reddit_username'] = re_reddit_user.match(
                url_dictionary['url']).group(5)
            log.debug(f"reddit user match: {url_dictionary['url']}")
            await self.reddit_user(url_dictionary.copy())

        elif re_subreddit.match(url_dictionary['url']):

            url_dictionary['subreddit'] = re_subreddit.match(
                url_dictionary['url']).group(5)

            # extra = top/?sort=top&t=week
            extra = re_subreddit.match(url_dictionary['url']).group(6)

            extra_info = {}

            if extra:
                _, extra = extra.split('?')

                for val in extra.split("&"):
                    k, v = val.split("=")
                    extra_info[k] = v
                # _, time_frame = extra.split("&")
                # extra_info['t'] = time_frame.split("=")[1]

            log.debug(f"subreddit match: {url_dictionary['url']}")
            await self.subreddit_download(url_dictionary.copy(), extra_info)

        else:
            log.info(
                f"No matching regex found for {url_dictionary['url']}")
            await self.common_failed_attempt(url_dictionary.copy())

    async def reddit_media(self, url_dictionary: UrlDictionary) -> None:

        log.debug("Inside reddit media download")
        if re_reddit_media.match(url_dictionary['url']).group(2) == 'i.':
            log.debug("reddit image found")

            user_agent = {'user-agent': self.user_agent}

            try:
                async with httpx.AsyncClient() as client:
                    response = await client.head(
                        url_dictionary['url'], headers=user_agent, timeout=None)

                if response.status_code >= 300:
                    await self.common_bad_status_code(url_dictionary.copy())
                    return

                temp = json.dumps(dict(response.headers))
                data = json.loads(temp)
                # print(data)

                url_dictionary['prefix'] = self.prefix

                url_dictionary['date'] = await self.common_get_epoch_time(
                    str(response.headers.get('last-modified')))

                url_dictionary['name'] = re_reddit_media.match(
                    url_dictionary['url']).group(6)

                url_dictionary['url_to_download'] = str(response.url)

                if data.get('Content-Length'):
                    url_dictionary['file_size'] = int(
                        data['Content-Length'])

                url_dictionary['extension'] = data['content-type']
                url_dictionary['filename'] = self.common_filename_creator(
                    url_dictionary)

                # await self.common_file_downloader(url_dictionary.copy())
                log.info("Url dictionary built. Sending to tracker")
                await self.tracker.add_url_dictionary(url_dictionary.copy())
                await self.common_advance_search_count(url_dictionary.copy())

            except Exception:
                log.exception(
                    f"Problem downloading file. Saving for later: {url_dictionary['url_to_download']}")
                await self.common_failed_attempt(url_dictionary.copy())

        if re_reddit_media.match(url_dictionary['url']).group(2) == 'v.':

            log.debug("Reddit video found")
            async with httpx.AsyncClient() as client:
                response = await client.get(
                    url_dictionary['url'], headers=self.headers, follow_redirects=True, timeout=None)

            # print(response.url)
            url_dictionary['url'] = str(response.url)

            try:
                url_dictionary['reddit_uniq_id'] = re_reddit_post.match(
                    url_dictionary['url']).group(7)

            except AttributeError:
                # print(url_dictionary['url'])
                url_dictionary['ytdlp_required'] = True
                url_dictionary['url_to_download'] = url_dictionary['url']

                url_dictionary['name'] = re_reddit_media.match(
                    url_dictionary['url']).group(6)

                url_dictionary['extension'] = 'mp4'

                url_dictionary['filename'] = self.common_filename_creator(
                    url_dictionary.copy())

                # print(url_dictionary)
                # await self.common_file_downloader(url_dictionary.copy())
                log.info("Url dictionary built. Sending to tracker")
                await self.tracker.add_url_dictionary(url_dictionary.copy())
                await self.common_advance_search_count(url_dictionary.copy())
                return

            await self.reddit_post(url_dictionary.copy())

    async def reddit_post(self, url_dictionary: UrlDictionary) -> None:

        endpoint = f"comments/{url_dictionary['reddit_uniq_id']}/.json"
        params = {}
        params["raw_json"] = "1"

        async with sem:
            log.debug("Calling reddit api for a post")
            data = await self.call(endpoint, params, url_dictionary.copy())

            try:

                post = data[0]['data']['children'][0]['data']

                if post['url'] != "":
                    await self.reddit_media_post(data, url_dictionary.copy())

                else:
                    await self.reddit_text_post(data, url_dictionary.copy())

            except TypeError:
                log.warn(
                    f"Servers are down or most likely user profile has been deleted.")
                await self.common_failed_attempt(url_dictionary.copy())

    async def reddit_text_post(self, data, url_dictionary: UrlDictionary) -> None:

        log.info("Found text post.")
        post = data[0]['data']['children'][0]['data']

        already_downloaded_urls = rat_info.get_downloaded_urls()

        if post['url'] in already_downloaded_urls:
            log.info("Text post already downloaded. Aborting save.")
            return

        if post['selftext'] == '':
            log.info("No text found in post")
            return

        log.info("Saving text post")
        url_dictionary['prefix'] = self.prefix
        url_dictionary['album_name'] = f"{post['subreddit']}-{url_dictionary['reddit_uniq_id']}"
        url_dictionary['name'] = post['title']
        url_dictionary['date'] = post['created']
        url_dictionary['extension'] = 'txt'
        filename = self.common_filename_creator(url_dictionary.copy())
        filename = filename.replace('.part', '')
        post_content = post['selftext']

        url_dictionary['progress']['search_message'].add_task(
            f"Reddit - Downloading text post for: {url_dictionary['name']}")

        with open(filename, 'w') as file:
            file.write(post_content)

        rat_info.add_entry(category_1='urls_downloaded', entry=post['url'])
        log.info("Text post saved")

    async def reddit_media_post(self, data, url_dictionary: UrlDictionary) -> None:

        try:

            post = data[0]['data']['children'][0]['data']
            url_dictionary['url_to_download'] = str(post['url'])

            log.info(
                f"Reddit media post found: {url_dictionary['url_to_download']}")

            if conductor.imgur_re.search(url_dictionary['url_to_download']):

                url_dictionary['url'] = url_dictionary['url_to_download']
                url_dictionary['url_to_record'] = url_dictionary['url_to_download']
                url_dictionary['album_name'] = url_dictionary['reddit_uniq_id']
                url_dictionary['description'] = str(post['title'])
                url_dictionary['date'] = int(post['created_utc'])
                await conductor.imgur(url_dictionary.copy())

            elif conductor.redgifs_re.search(url_dictionary['url_to_download']):

                url_dictionary['url'] = url_dictionary['url_to_download']
                url_dictionary['album_name'] = url_dictionary['reddit_uniq_id']
                url_dictionary['description'] = str(post['title'])
                log.debug("redgifs link found, sending to conductor")
                await conductor.redgifs(url_dictionary.copy())

            elif conductor.gfycat_re.search(url_dictionary['url_to_download']):

                url_dictionary['url'] = url_dictionary['url_to_download']
                url_dictionary['album_name'] = url_dictionary['reddit_uniq_id']
                url_dictionary['description'] = str(post['title'])
                log.debug("gfycat link found, sending to conductor")
                await conductor.gfycat(url_dictionary.copy())

            elif post.get('is_gallery'):

                if post.get('gallery_data'):
                    await self.reddit_gallery_download(post, url_dictionary)

                else:
                    log.info(
                        f"Post Deleted: {url_dictionary['reddit_uniq_id']}")
                    await self.common_failed_attempt(url_dictionary.copy())
                    return

            elif re_reddit_media.match(post['url']).group(2) == "i.":

                url_dictionary['url'] = str(post['url'])
                url_dictionary['description'] = str(post['title'])
                url_dictionary['album_name'] = url_dictionary['reddit_uniq_id']

                await self.reddit_media(url_dictionary.copy())

            elif re_reddit_media.match(post['url']).group(2) == "v.":
                await self.reddit_video_download(post, url_dictionary.copy())

            else:
                log.warn(
                    f"No regex match found for: {url_dictionary['url_to_download']}. \
                    Seeing if url matches a pattern in conductor")
                await conductor.validate_url(url_dictionary.copy())
                return

        except AttributeError:

            log.error(
                f"Not a downloadable link: {url_dictionary['url_to_download']}. Attempting text post download")
            await self.reddit_text_post(data, url_dictionary.copy())
            return
            # await self.common_failed_attempt(url_dictionary.copy())

        except KeyError:
            log.error("Problem downloading. Reddit servers might be down")
            await self.common_failed_attempt(url_dictionary.copy())
            return

        except Exception as e:
            log.exception(e)
            return

    async def reddit_user(self, url_dictionary: UrlDictionary) -> None:

        log.info(f"Reddit user found: {url_dictionary['reddit_username']}")
        endpoint = f"user/{url_dictionary['reddit_username']}/submitted/.json"
        params = {}

        tasks = []
        already_downloaded_urls = rat_info.get_downloaded_urls()
        end = False
        while end is False:

            log.debug("Calling Reddit api to get user info")

            params["limit"] = 100
            data = await self.call(endpoint, params, url_dictionary.copy())
            log.debug("Received Reddit user data")
            # log.debug(data['data']['children'])

            try:
                for post in data['data']['children']:

                    if post['data']['url'] in already_downloaded_urls:
                        continue

                    elif post['data']['url'] not in already_downloaded_urls:
                        url_dictionary['reddit_uniq_id'] = post['data']['id']
                        tasks.append(asyncio.create_task(
                            self.reddit_post(url_dictionary.copy())))

                if data['data']['after']:
                    params['after'] = data['data']['after']

                else:
                    end = True

            except TypeError:
                log.warn(
                    f"Servers could be down, but most likely user profile has been deleted. Reddit username: {url_dictionary['reddit_username']}")
                end = True

        await asyncio.gather(*tasks)

    async def subreddit_download(self, url_dictionary: UrlDictionary, extra_info: dict[str, str]) -> None:

        log.info(f"Subreddit found: {url_dictionary['subreddit']}")
        max_posts_to_get = 0
        params = {}
        endpoint = f"r/{url_dictionary['subreddit']}"
        if extra_info:

            params['t'] = extra_info['t']

            endpoint = f"{endpoint}/{extra_info['sort']}.json"

            if extra_info.get('limit'):
                max_posts_to_get = int(extra_info['limit'])

            else:
                max_posts_to_get = 0

        else:
            endpoint = endpoint + "/.json"

        tasks = []
        already_downloaded_urls = rat_info.get_downloaded_urls()
        count = 0
        posts_gotten = 0
        end = False
        while end is False:

            log.debug("Calling Reddit api to get subreddit info")

            if 0 < max_posts_to_get < 100:
                params["limit"] = max_posts_to_get

            else:
                params["limit"] = 100

            data = await self.call(endpoint, params, url_dictionary.copy())
            log.debug("Received subreddit data")
            # log.debug(data['data']['children'])

            try:
                for post in data['data']['children']:

                    try:
                        count += 1
                        if post['data']['url'] in already_downloaded_urls:

                            posts_gotten += 1

                            if max_posts_to_get != 0 and posts_gotten >= max_posts_to_get:
                                break
                            else:
                                continue

                        url_dictionary['reddit_uniq_id'] = post['data']['id']
                        url_dictionary['count'] = count
                        tasks.append(asyncio.create_task(
                            self.reddit_post(url_dictionary.copy())))

                        posts_gotten += 1

                        if max_posts_to_get != 0 and posts_gotten >= max_posts_to_get:
                            break

                    except TypeError:
                        log.warn(f"Post Most Likely Deleted")
                        continue

                if max_posts_to_get != 0 and posts_gotten >= max_posts_to_get:
                    break

                if data['data']['after']:
                    params['after'] = data['data']['after']

                else:
                    end = True

            except TypeError:
                log.warn(
                    f"Servers could be down, but most likely post has been deleted.")
                end = True

        await asyncio.gather(*tasks)

    async def reddit_gallery_download(self, post, url_dictionary):

        log.debug("gallery found")
        image_order = []
        for entry in post['gallery_data']['items']:
            id = entry['media_id']
            image_order.append(id)

        count = 0
        try:
            for id in image_order:

                count += 1

                image_link = post['media_metadata'][id]['p'][0]['u']

                url_dictionary['name'] = re_reddit_gallery_link.match(
                    image_link).group(4)

                url_dictionary['extension'] = re_reddit_gallery_link.match(
                    image_link).group(5)

                image_url = f"https://i.redd.it/{url_dictionary['name']}.{url_dictionary['extension']}"

                async with httpx.AsyncClient() as client:
                    response = await client.head(
                        image_url, headers=self.headers, timeout=None)

                temp = json.dumps(dict(response.headers))
                data = json.loads(temp)

                url_dictionary['prefix'] = self.prefix
                url_dictionary['date'] = post['created_utc']

                url_dictionary['album_name'] = url_dictionary['reddit_uniq_id']
                url_dictionary['count'] = count
                url_dictionary['description'] = str(post['title'])

                if data.get('content-length'):
                    url_dictionary['file_size'] = int(data['content-length'])

                url_dictionary['url_to_download'] = image_url
                url_dictionary['filename'] = self.common_filename_creator(
                    url_dictionary.copy())

                log.info("Url dictionary built. Sending to tracker")
                await self.tracker.add_url_dictionary(url_dictionary.copy())
                await self.common_advance_search_count(url_dictionary.copy())

        except IndexError:
            pass

    async def reddit_video_download(self, post, url_dictionary):

        log.info(f"Reddit video found: {post['url']}")

        url_dictionary['ytdlp_required'] = True
        url_dictionary['prefix'] = self.prefix
        url_dictionary['date'] = post['created_utc']
        url_dictionary['name'] = url_dictionary['reddit_uniq_id']
        url_dictionary['description'] = post['title']
        url_dictionary['url_to_download'] = str(post['url'])
        url_dictionary['extension'] = "mp4"
        url_dictionary['filename'] = self.common_filename_creator(
            url_dictionary.copy())

        # await self.common_file_downloader(url_dictionary.copy())
        log.info("Url dictionary built. Sending to tracker")
        await self.tracker.add_url_dictionary(url_dictionary.copy())
        await self.common_advance_search_count(url_dictionary.copy())



import asyncio
import logging


from ripandtear.utils.progress import file_category_progress
from ripandtear.utils import rat_info

from ripandtear.utils import downloader

log = logging.getLogger(__name__)


class Tracker:

    __instance = None

    @staticmethod
    def getInstance():
        if Tracker.__instance is None:
            Tracker()
        return Tracker.__instance

    def __init__(self):
        Tracker.__instance = self
        self.url_dictionaries = []
        self.fast_dictionaries = []
        self.medium_dictionaries = []
        self.slow_dictionaries = []

    async def add_url_dictionary(self, url_dictionary):
        log.debug("Adding url_dictionary")
        self.url_dictionaries.append(url_dictionary)

    def total_url_dictionaries(self):
        return len(self.url_dictionaries)

    async def print_urls(self):
        for url_dictionary in self.url_dictionaries:
            if url_dictionary.get('fail') is None:
                print(url_dictionary['url_to_download'])

    def clean_duplicates(self):
        log.info("Cleaning Duplicate Dictionaries")
        log.info(f"Before cleaning: {len(self.url_dictionaries)}")

        temp_dictionary = {}

        for url_dictionary in self.url_dictionaries:
            if url_dictionary.get("url_to_download"):
                url_to_download = url_dictionary["url_to_download"]

                if url_to_download not in temp_dictionary:
                    temp_dictionary[url_to_download] = url_dictionary
                else:
                    try:
                        saved_filename = temp_dictionary[url_to_download]["filename"]
                        new_filename = url_dictionary["filename"]

                        if len(new_filename) > len(saved_filename):
                            temp_dictionary[url_to_download] = url_dictionary
                    except KeyError:
                        continue

        deduplicated_dictionaries = []

        for k, v in temp_dictionary.items():
            deduplicated_dictionaries.append(v)

        self.url_dictionaries = deduplicated_dictionaries

        log.info(f"After cleaning: {len(self.url_dictionaries)}")

    async def download_url_dictionaries(self):
        self.clean_duplicates()
        log.info("Preparing downloads")

        failed_id = file_category_progress.add_task(
            "Failed", total=self.total_url_dictionaries())

        completed_id = file_category_progress.add_task(
            "Completed", total=self.total_url_dictionaries())

        downloaded_id = file_category_progress.add_task(
            "Downloaded", total=self.total_url_dictionaries())

        fast_semaphore = asyncio.Semaphore(6)
        medium_semaphore = asyncio.Semaphore(4)
        slow_semaphore = asyncio.Semaphore(2)

        already_downloaded_urls = rat_info.get_downloaded_urls()

        for url_dictionary in self.url_dictionaries:

            url_dictionary['progress'] = {}
            url_dictionary['progress']['failed_id'] = failed_id
            url_dictionary['progress']['completed_id'] = completed_id
            url_dictionary['progress']['downloaded_id'] = downloaded_id

            # This is to handle cases where an error dictionary might not
            # have set the url_to_download before encountering an error,
            # but set the correct url to ['url']

            if url_dictionary.get('url_to_download') is None:
                url_dictionary['url_to_download'] = url_dictionary['url']
                continue

            if url_dictionary.get('fail'):
                rat_info.add_error_dictionary(url_dictionary.copy())
                file_category_progress.advance(
                    url_dictionary['progress']['failed_id'], 1)
                continue

            if url_dictionary.get('completed'):
                file_category_progress.advance(
                    url_dictionary['progress']['completed_id'], 1)
                continue

            log.debug("Checking if url has already been downloaded")
            if url_dictionary.get('url_to_record'):
                if url_dictionary['url_to_record'] in already_downloaded_urls:
                    log.info(
                        f"Url already recorded: {url_dictionary['url_to_record']}")
                    file_category_progress.advance(
                        url_dictionary['progress']['completed_id'], 1)
                    continue

            if url_dictionary['url_to_download'] in already_downloaded_urls:
                log.info(
                    f"Url already recorded: {url_dictionary['url_to_download']}")
                file_category_progress.advance(
                    url_dictionary['progress']['completed_id'], 1)
                continue

            log.debug("url has not been downloaded yet")

            if 'media-files' in url_dictionary['url_to_download']:
                self.slow_dictionaries.append(asyncio.create_task(
                    downloader.download_file(url_dictionary.copy(), slow_semaphore)))
                continue

            if 'coomer' in url_dictionary['url_to_download']:
                self.slow_dictionaries.append(asyncio.create_task(
                    downloader.download_file(url_dictionary.copy(), slow_semaphore)))
                continue

            if 'cyberdrop' in url_dictionary['url_to_download']:
                self.medium_dictionaries.append(asyncio.create_task(
                    downloader.download_file(url_dictionary.copy(), medium_semaphore)))
                continue

            self.fast_dictionaries.append(asyncio.create_task(
                downloader.download_file(url_dictionary.copy(), fast_semaphore)))

        if len(self.fast_dictionaries) >= 1:
            log.info("Starting Fast Downloads")
            await asyncio.gather(*self.fast_dictionaries)

        if len(self.medium_dictionaries) >= 1:
            log.info("Starting Medium Downloads")
            await asyncio.gather(*self.medium_dictionaries)

        if len(self.slow_dictionaries) >= 1:
            log.info("Starting Slow Downloads")
            await asyncio.gather(*self.slow_dictionaries)



from pathlib import Path

from rich import padding
from rich.panel import Panel
from rich.table import Table
from rich.progress import Progress, SpinnerColumn, DownloadColumn, TextColumn, TransferSpeedColumn, TimeRemainingColumn, BarColumn
from rich.style import Style

dir_name = Path.cwd().name

search_message = Progress(TextColumn("{task.description}"))

search_info = Progress(SpinnerColumn(),
                       TextColumn("[progress.description]{task.description}"),
                       BarColumn(bar_width=None),
                       "{task.completed}")

file_category_progress = Progress("{task.description}",
                                  BarColumn(bar_width=None,
                                            complete_style='chartreuse2'),
                                  "[progress.percentage]{task.percentage:>3.1f}%",
                                  "[progress.completed]{task.completed} out of {task.total}")

file_progress = Progress(SpinnerColumn(),
                         "{task.description}",
                         BarColumn(bar_width=None,
                                   complete_style=Style(color='chartreuse2')),
                         "[progress.percentage]{task.percentage:>3.1f}%",
                         "-",
                         DownloadColumn(),
                         "-",
                         TransferSpeedColumn(),
                         "-",
                         TimeRemainingColumn(),)


async def search_table():

    search_table = Table.grid(expand=True)

    search_table.add_row(Panel.fit(
        search_message, title="Search Messages", border_style='green', padding=(1, 1)))

    search_table.add_row(Panel.fit(
        search_info, title=f"Searching for {dir_name}", border_style='green', padding=(1, 1)))

    return search_table


async def download_table():

    progress_table = Table.grid(expand=True)

    progress_table.add_row(Panel.fit(
        file_category_progress, title=f"Downloading {dir_name}", border_style='green', padding=(1, 1)))

    progress_table.add_row(
        Panel.fit(file_progress, title="Files Being Downloaded", border_style='green', padding=(1, 1)))
    return progress_table



import asyncio
import httpx
import logging
import re

from playwright.async_api import async_playwright
from bs4 import BeautifulSoup

from ripandtear.extractors.common import Common
from ripandtear.utils.custom_types import UrlDictionary
from ripandtear.utils import rat_info
from ripandtear.utils.tracker import Tracker

log = logging.getLogger(__name__)


prefix = "redgifs"
url_root = "https://www.redgifs.com/"
url_root_v3 = "https://v3.redgifs.com/"

# https://api.redgifs.com/v2/gifs/firstscientificloon
single_uid_api = "https://api.redgifs.com/v2/gifs/"

re_tag_url = re.compile(r"(https?://)(\w+\.)(redgifs\.com/)gifs\/(.*)")

re_extension = re.compile(
    r"(https?://)(\w+\.)(redgifs.com/)(\w+)(-\w+)?\.(\w+)")

re_watch_uid = re.compile(
    r"(/watch/)(\w+)#")

# 'https://i.redgifs.com/i/kosherpurehairstreak.jpg'
# https://v3.redgifs.com/watch/heavyimperfectkookaburra#rel=user%3Alamsinka89
# https://www.redgifs.com/watch/firstscientificloon (album with multiple images)
re_single_uid = re.compile(
    r"(https?://)(\w+\.)?(redgifs.com/)(watch|i|ifr)/(\w+)")

re_url_to_record = re.compile(
    r"((https?://)(\w+\.)?(redgifs.com/)(watch|i|ifr)/(\w+))")

# https://v3.redgifs.com/users/lamsinka89
re_user = re.compile(
    r"((https?://)(\w+\.)?(redgifs\.com)/(users)/([\w\-]+))")

sem = asyncio.Semaphore(6)


class Redgifs(Common):

    def __init__(self):
        self.tracker = Tracker.getInstance()

    async def get_token(self) -> None:

        log.debug("Requesting auth token")
        headers = {'token': ''}

        async with httpx.AsyncClient() as client:
            response = await client.get(
                "https://api.redgifs.com/v2/auth/temporary", headers=headers, timeout=None)

        data = response.json()

        self.headers = {'authorization': f"Bearer {data['token']}"}
        # print(self.headers)

        log.debug("Auth token set")

    async def run(self, url_dictionary: UrlDictionary) -> None:

        await self.get_token()

        if re_single_uid.match(url_dictionary['url']):

            log.debug("Single image found. Sending to single single_download")
            await self.single_download(re_single_uid.match(
                url_dictionary['url']).group(5), url_dictionary.copy())

        elif re_user.match(url_dictionary['url']):

            log.debug("User found. Sending to user_download_v3")
            await self.user_download_v3(re_user.match(
                url_dictionary['url']).group(6), url_dictionary.copy())

        elif re_tag_url.match(url_dictionary['url']):

            log.debug("Tags url found. Preparing download")
            await self.tag_download(url_dictionary.copy())

    async def tag_download(self, url_dictionary: UrlDictionary) -> None:

        log.info("Found Tag. Preparing download")

        async with async_playwright() as pw:

            try:
                browser = await pw.chromium.launch()
                page = await browser.new_page()
                await page.goto(url_dictionary['url'])

                log.info("Searching for videos")
                content = await page.query_selector_all('a.tile')

                while True:

                    for x in range(5):
                        await page.keyboard.down('PageDown')
                        await page.wait_for_timeout(200)

                    new_content = await page.query_selector_all('a.tile')

                    if len(new_content) > len(content):

                        content = new_content
                        log.info(f"Gifs found: {len(new_content)}")

                    else:
                        content = new_content
                        break

                if page.locator('div.gifList'):
                    html = await page.locator('div.gifList').inner_html()

                elif page.locator('div.tileFeed'):
                    html = await page.locator('div.tileFeed').inner_html()

                soup = BeautifulSoup(html, features='html.parser')

                already_downloaded_urls = rat_info.get_downloaded_urls()

                tasks = []
                for link in soup.find_all('a'):

                    href = link.get('href').split('#')[0]

                    url = f"https://v3.redgifs.com{href}"

                    if url in already_downloaded_urls and url_dictionary['download'] is True:
                        log.info(f"Already downloaded: {url}")
                        continue

                    elif url not in already_downloaded_urls and url_dictionary['download'] is True:
                        log.info(f"Creating task out of {url}")

                        url_dictionary['url'] = url
                        tasks.append(asyncio.create_task(
                            self.run(url_dictionary.copy())))

                    elif url_dictionary['download'] is False:
                        url_dictionary['url_to_download'] = url
                        continue

                log.info("Executing tasks")
                await asyncio.gather(*tasks)

            except Exception:
                log.exception("Problem getting gifs from tag url")
                await self.common_failed_attempt(url_dictionary.copy())

            finally:
                await page.close()
                await browser.close()

    async def single_download(self, uid: str, url_dictionary: UrlDictionary, download_gallery=True) -> None:
        # some urls can contain albums that need to use playwright to get all image uid's because albums
        # are not supported by the API. If the API returns the first image and in the data it says that
        # the image is part of a gallery, playwright opens the webpage to find all the individual images
        # within the album. It then recursivly sends it back to teh single_download to download all images
        # because each of those images say that they are apart of a gallery it can cause a forever loop.
        # the download_gallery variable is used to preven a forever loop. When album links are found
        # with playwright the variable is set to False to skip the playwright download a second time and
        # download the individual image

        async with sem:

            log.debug("Single image given. Attempting download")

            async with httpx.AsyncClient() as client:
                response = await client.get(
                    f"{single_uid_api}{uid}", headers=self.headers, timeout=None)

            data = response.json()
            # print(data)

            if 'error' in data:

                log.info(f"File Deleted: {url_dictionary['url']}")
                return

            elif data['gif']['gallery'] is not None and download_gallery is True:
                async with async_playwright() as pw:

                    try:
                        browser = await pw.chromium.launch()
                        page = await browser.new_page()
                        await page.goto(f"https://www.redgifs.com/watch/{uid}")

                        log.info("Searching for album")
                        content = await page.locator("div.GallerySlider").inner_html()
                        # print(content)

                        log.info("Album found. Finding individual images")
                        soup = BeautifulSoup(content, features='html.parser')
                        # print(soup)

                        already_downloaded_urls = rat_info.get_downloaded_urls()

                        tasks = []
                        count = 0
                        for link in soup.find_all('img'):

                            href = link.get('src')

                            new_uid = re_extension.match(href).group(4).lower()
                            url = f"https://v3.redgifs.com/watch/{new_uid}"

                            if url in already_downloaded_urls and url_dictionary['download'] is True:
                                log.info(f"Already downloaded: {url}")
                                continue

                            elif url not in already_downloaded_urls and url_dictionary['download'] is True:
                                log.info(f"Creating task out of {new_uid}")

                                url_dictionary['url'] = url
                                count += 1
                                url_dictionary['count'] = count
                                url_dictionary['album_name'] = uid
                                tasks.append(asyncio.create_task(
                                    self.single_download(new_uid, url_dictionary.copy(), download_gallery=False)))

                            elif url_dictionary['download'] is False:
                                url_dictionary['url_to_download'] = url
                                continue

                        log.info("All images in album found. Executing tasks")
                        await asyncio.gather(*tasks)

                    except Exception:
                        log.exception("Problem getting gifs from album")
                        await self.common_failed_attempt(url_dictionary.copy())

                    finally:
                        await page.close()
                        await browser.close()

            else:

                url_dictionary['prefix'] = prefix
                url_dictionary['name'] = data['gif']['id']
                url_dictionary['date'] = data['gif']['createDate']
                url_dictionary['url_to_download'] = data['gif']['urls']['hd']
                url_dictionary['url_to_record'] = re_url_to_record.match(
                    url_dictionary['url']).group(1)

                url_dictionary['extension'] = re_extension.match(
                    url_dictionary['url_to_download']).group(6)

                url_dictionary['filename'] = self.common_filename_creator(
                    url_dictionary)

                log.info("Sending dictionary to tracker")
                await self.tracker.add_url_dictionary(url_dictionary.copy())
                await self.common_advance_search_count(url_dictionary.copy())

    async def user_download_v3(self, username: str, url_dictionary: UrlDictionary) -> None:

        log.info(f"Redgifs user Found: {username}")

        user_url = f"https://v3.redgifs.com/users/{username}"

        async with async_playwright() as pw:

            try:
                browser = await pw.chromium.launch()
                page = await browser.new_page()
                await page.goto(user_url)

                log.info("Searching for videos")
                content = await page.query_selector_all('a.tile')

                while True:

                    for x in range(5):
                        await page.keyboard.down('PageDown')
                        await page.wait_for_timeout(200)

                    new_content = await page.query_selector_all('a.tile')

                    if len(new_content) > len(content):

                        content = new_content
                        log.info(f"Gifs found: {len(new_content)}")

                    else:
                        content = new_content
                        break

                if page.locator('div.gifList'):
                    html = await page.locator('div.gifList').inner_html()

                elif page.locator('div.tileFeed'):
                    html = await page.locator('div.tileFeed').inner_html()

                soup = BeautifulSoup(html, features='html.parser')

                already_downloaded_urls = rat_info.get_downloaded_urls()

                tasks = []
                for link in soup.find_all('a'):

                    href = link.get('href')

                    uid = re_watch_uid.match(href).group(2)
                    url = f"https://v3.redgifs.com/watch/{uid}"

                    if url in already_downloaded_urls and url_dictionary['download'] is True:
                        log.info(f"Already downloaded: {url}")
                        continue

                    elif url not in already_downloaded_urls and url_dictionary['download'] is True:
                        log.info(f"Creating task out of {uid}")

                        url_dictionary['url'] = url
                        tasks.append(asyncio.create_task(
                            self.single_download(uid, url_dictionary.copy())))

                    elif url_dictionary['download'] is False:
                        url_dictionary['url_to_download'] = url
                        continue

                log.info("Executing tasks")
                await asyncio.gather(*tasks)

            except Exception:
                log.exception("Problem getting gifs from user profile")
                await self.common_failed_attempt(url_dictionary.copy())

            finally:
                await page.close()
                await browser.close()



import asyncio
import logging
import re

from ripandtear.extractors.common import Common
from ripandtear.utils.custom_types import UrlDictionary
from ripandtear.utils.tracker import Tracker

log = logging.getLogger(__name__)

# Errors
# 'https://i.imgur.com/LKvqXWE.gifv'

prefix = 'imgur'

imgur_re_direct_image = re.compile(
    r"(https?://)([im]\.|www\.)?(imgur\.(com|io))/(\w+)(\.jpg|jpeg|png|gifv|mp4|gif)?")
# 'https://i.imgur.com/I7i8u6b.gifv'
# 'https://i.imgur.com/hCOoORb.png'

imgur_re_album = re.compile(
    r"(https?://)?(([im]\.)|(www.))?(imgur\.(com|io))/((a))/(\w+)")
# 'https://imgur.com/a/1GXfC'
# 'https://imgur.com/a/1GXfC/all'

imgur_re_gallery = re.compile(
    r"(https?://)(imgur\.(com|io))/gallery/(\w+)")
# 'https://imgur.com/gallery/mPIxZnC'


class Imgur(Common):

    def __init__(self):

        self.tracker = Tracker.getInstance()

    def imgur_direct_image_hash(self, url: str) -> str:
        log.debug("Looking for direct image hash")
        direct_image_hash = imgur_re_direct_image.search(url).group(5)
        return direct_image_hash

    async def run(self, url_dictionary: UrlDictionary) -> None:

        if imgur_re_direct_image.match(url_dictionary['url']):

            log.debug("Direct image found. Sending to single download")
            await self.imgur_single_download(self.imgur_direct_image_hash(url_dictionary['url']),
                                             url_dictionary.copy())

        else:
            log.info(f"No regex found for {url_dictionary['url']}")

    async def imgur_single_download(self, image_hash: str, url_dictionary: UrlDictionary) -> None:

        try:
            url_dictionary['prefix'] = prefix
            url_dictionary['headers'] = {
                'user-agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36'}
            url_dictionary['url_to_download'] = url_dictionary['url']

            if 'gifv' in url_dictionary['url_to_download']:
                url_dictionary['url_to_download'] = url_dictionary['url_to_download'].replace(
                    'gifv', 'mp4')

            url_dictionary['name'] = image_hash
            url_dictionary['extension'] = self.common_find_url_extension(
                url_dictionary['url_to_download'])
            url_dictionary['filename'] = self.common_filename_creator(
                url_dictionary.copy())

            log.info("Url dictionary built. Sending to tracker")
            await self.tracker.add_url_dictionary(url_dictionary.copy())
            await self.common_advance_search_count(url_dictionary.copy())

        except Exception:
            log.error(
                f"Problem downloading image: {url_dictionary['url_to_download']}")
